@@ -11,10 +11,11 @@ import numpy as np
 import scipy.integrate as si
 import scipy.optimize as so
 
-from .tools import (_greater_or_close, _less_or_close, broadcast_indices,
-                    find_bounding_indices, find_intersections, first_derivative, get_layer,
-                    interp)
+from .tools import (_greater_or_close, _less_or_close, find_bounding_indices,
+                    find_intersections, first_derivative, get_layer)
+from ..cbook import broadcast_indices
 from ..constants import Cp_d, epsilon, g, kappa, Lv, P0, Rd
+from ..interpolate.one_dimension import interpolate_1d
 from ..package_tools import Exporter
 from ..units import atleast_1d, check_units, concatenate, units
 from ..xarray import preprocess_xarray
@@ -349,8 +350,11 @@ def lfc(pressure, temperature, dewpt, parcel_temperature_profile=None):
     """
     # Default to surface parcel if no profile or starting pressure level is given
     if parcel_temperature_profile is None:
-        parcel_temperature_profile = parcel_profile(pressure, temperature[0],
-                                                    dewpt[0]).to('degC')
+        new_stuff = parcel_profile_with_lcl(pressure, temperature, dewpt)
+        pressure, temperature, _, parcel_temperature_profile = new_stuff
+        temperature = temperature.to('degC')
+        parcel_temperature_profile = parcel_temperature_profile.to('degC')
+
     # The parcel profile and data have the same first data point, so we ignore
     # that point to get the real first intersection for the LFC calculation.
     x, y = find_intersections(pressure[1:], parcel_temperature_profile[1:],
@@ -412,8 +416,11 @@ def el(pressure, temperature, dewpt, parcel_temperature_profile=None):
     """
     # Default to surface parcel if no profile or starting pressure level is given
     if parcel_temperature_profile is None:
-        parcel_temperature_profile = parcel_profile(pressure, temperature[0],
-                                                    dewpt[0]).to('degC')
+        new_stuff = parcel_profile_with_lcl(pressure, temperature, dewpt)
+        pressure, temperature, _, parcel_temperature_profile = new_stuff
+        temperature = temperature.to('degC')
+        parcel_temperature_profile = parcel_temperature_profile.to('degC')
+
     # If the top of the sounding parcel is warmer than the environment, there is no EL
     if parcel_temperature_profile[-1] > temperature[-1]:
         return np.nan * pressure.units, np.nan * temperature.units
@@ -455,27 +462,99 @@ def parcel_profile(pressure, temperature, dewpt):
     --------
     lcl, moist_lapse, dry_lapse
 
+    """
+    _, _, _, t_l, _, t_u = _parcel_profile_helper(pressure, temperature, dewpt)
+    return concatenate((t_l, t_u))
+
+
+@exporter.export
+@preprocess_xarray
+@check_units('[pressure]', '[temperature]', '[temperature]')
+def parcel_profile_with_lcl(pressure, temperature, dewpt):
+    r"""Calculate the profile a parcel takes through the atmosphere.
+
+    The parcel starts at `temperature`, and `dewpt`, lifted up
+    dry adiabatically to the LCL, and then moist adiabatically from there.
+    `pressure` specifies the pressure levels for the profile. This function returns
+    a profile that includes the LCL.
+
+    Parameters
+    ----------
+    pressure : `pint.Quantity`
+        The atmospheric pressure level(s) of interest. The first entry should be the starting
+        point pressure.
+    temperature : `pint.Quantity`
+        The atmospheric temperature at the levels in `pressure`. The first entry should be the
+        starting point temperature.
+    dewpt : `pint.Quantity`
+        The atmospheric dew point at the levels in `pressure`. The first entry should be the
+        starting dew point.
+
+    Returns
+    -------
+    pressure : `pint.Quantity`
+        The parcel profile pressures, which includes the specified levels and the LCL
+    ambient_temperature : `pint.Quantity`
+        The atmospheric temperature values, including the value interpolated to the LCL level
+    ambient_dew_point : `pint.Quantity`
+        The atmospheric dew point values, including the value interpolated to the LCL level
+    profile_temperature : `pint.Quantity`
+        The parcel profile temperatures at all of the levels in the returned pressures array,
+        including the LCL.
+
+    See Also
+    --------
+    lcl, moist_lapse, dry_lapse, parcel_profile
+
+    """
+    p_l, p_lcl, p_u, t_l, t_lcl, t_u = _parcel_profile_helper(pressure, temperature[0],
+                                                              dewpt[0])
+    new_press = concatenate((p_l, p_lcl, p_u))
+    prof_temp = concatenate((t_l, t_lcl, t_u))
+    new_temp = _insert_lcl_level(pressure, temperature, p_lcl)
+    new_dewp = _insert_lcl_level(pressure, dewpt, p_lcl)
+    return new_press, new_temp, new_dewp, prof_temp
+
+
+def _parcel_profile_helper(pressure, temperature, dewpt):
+    """Help calculate parcel profiles.
+
+    Returns the temperature and pressure, above, below, and including the LCL. The
+    other calculation functions decide what to do with the pieces.
+
     """
     # Find the LCL
-    lcl_pressure, _ = lcl(pressure[0], temperature, dewpt)
-    lcl_pressure = lcl_pressure.to(pressure.units)
+    press_lcl, temp_lcl = lcl(pressure[0], temperature, dewpt)
+    press_lcl = press_lcl.to(pressure.units)
 
     # Find the dry adiabatic profile, *including* the LCL. We need >= the LCL in case the
     # LCL is included in the levels. It's slightly redundant in that case, but simplifies
     # the logic for removing it later.
-    press_lower = concatenate((pressure[pressure >= lcl_pressure], lcl_pressure))
-    t1 = dry_lapse(press_lower, temperature)
+    press_lower = concatenate((pressure[pressure >= press_lcl], press_lcl))
+    temp_lower = dry_lapse(press_lower, temperature)
 
     # If the pressure profile doesn't make it to the lcl, we can stop here
-    if _greater_or_close(np.nanmin(pressure), lcl_pressure.m):
-        return t1[:-1]
+    if _greater_or_close(np.nanmin(pressure), press_lcl.m):
+        return (press_lower[:-1], press_lcl, np.array([]) * press_lower.units,
+                temp_lower[:-1], temp_lcl, np.array([]) * temp_lower.units)
 
     # Find moist pseudo-adiabatic profile starting at the LCL
-    press_upper = concatenate((lcl_pressure, pressure[pressure < lcl_pressure]))
-    t2 = moist_lapse(press_upper, t1[-1]).to(t1.units)
+    press_upper = concatenate((press_lcl, pressure[pressure < press_lcl]))
+    temp_upper = moist_lapse(press_upper, temp_lower[-1]).to(temp_lower.units)
 
-    # Return LCL *without* the LCL point
-    return concatenate((t1[:-1], t2[1:]))
+    # Return profile pieces
+    return (press_lower[:-1], press_lcl, press_upper[1:],
+            temp_lower[:-1], temp_lcl, temp_upper[1:])
+
+
+def _insert_lcl_level(pressure, temperature, lcl_pressure):
+    """Insert the LCL pressure into the profile."""
+    interp_temp = interpolate_1d(lcl_pressure, pressure, temperature)
+
+    # Pressure needs to be increasing for searchsorted, so flip it and then convert
+    # the index back to the original array
+    loc = pressure.size - pressure[::-1].searchsorted(lcl_pressure)
+    return np.insert(temperature.m, loc, interp_temp.m) * temperature.units
 
 
 @exporter.export
@@ -1456,6 +1535,7 @@ def isentropic_interpolation(theta_levels, pressure, temperature, *args, **kwarg
 
     slices = [np.newaxis] * ndim
     slices[axis] = slice(None)
+    slices = tuple(slices)
     pres = np.broadcast_to(pres[slices], temperature.shape) * pres.units
 
     # Sort input data
@@ -1525,7 +1605,8 @@ def isentropic_interpolation(theta_levels, pressure, temperature, *args, **kwarg
 
     # do an interpolation for each additional argument
     if args:
-        others = interp(isentlevels, pres_theta.m, *(arr[sorter] for arr in args), axis=axis)
+        others = interpolate_1d(isentlevels, pres_theta.m, *(arr[sorter] for arr in args),
+                                axis=axis)
         if len(args) > 1:
             ret.extend(others)
         else:
@@ -1568,8 +1649,8 @@ def surface_based_cape_cin(pressure, temperature, dewpoint):
     cape_cin, parcel_profile
 
     """
-    profile = parcel_profile(pressure, temperature[0], dewpoint[0])
-    return cape_cin(pressure, temperature, dewpoint, profile)
+    p, t, td, profile = parcel_profile_with_lcl(pressure, temperature, dewpoint)
+    return cape_cin(p, t, td, profile)
 
 
 @exporter.export
@@ -2050,7 +2131,7 @@ def wet_bulb_temperature(pressure, temperature, dewpoint):
 
     This function calculates the wet-bulb temperature using the Normand method. The LCL is
     computed, and that parcel brought down to the starting pressure along a moist adiabat.
-    Norman method (and others) described and compared by [Knox2017]_.
+    The Normand method (and others) are described and compared by [Knox2017]_.
 
     Parameters
     ----------
@@ -2155,3 +2236,92 @@ def dewpoint_from_specific_humidity(specific_humidity, temperature, pressure):
     return dewpoint_rh(temperature, relative_humidity_from_specific_humidity(specific_humidity,
                                                                              temperature,
                                                                              pressure))
+
+
+@exporter.export
+@preprocess_xarray
+@check_units('[length]/[time]', '[pressure]', '[temperature]')
+def vertical_velocity_pressure(w, pressure, temperature, mixing=0):
+    r"""Calculate omega from w assuming hydrostatic conditions.
+
+    This function converts vertical velocity with respect to height
+    :math:`\left(w = \frac{Dz}{Dt}\right)` to that
+    with respect to pressure :math:`\left(\omega = \frac{Dp}{Dt}\right)`
+    assuming hydrostatic conditions on the synoptic scale.
+    By Equation 7.33 in [Hobbs2006]_,
+
+    .. math: \omega \simeq -\rho g w
+
+    Density (:math:`\rho`) is calculated using the :func:`density` function,
+    from the given pressure and temperature. If `mixing` is given, the virtual
+    temperature correction is used, otherwise, dry air is assumed.
+
+    Parameters
+    ----------
+    w: `pint.Quantity`
+        Vertical velocity in terms of height
+    pressure: `pint.Quantity`
+        Total atmospheric pressure
+    temperature: `pint.Quantity`
+        Air temperature
+    mixing: `pint.Quantity`, optional
+        Mixing ratio of air
+
+    Returns
+    -------
+    `pint.Quantity`
+        Vertical velocity in terms of pressure (in Pascals / second)
+
+    See Also
+    --------
+    density, vertical_velocity
+
+    """
+    rho = density(pressure, temperature, mixing)
+    return (- g * rho * w).to('Pa/s')
+
+
+@exporter.export
+@preprocess_xarray
+@check_units('[pressure]/[time]', '[pressure]', '[temperature]')
+def vertical_velocity(omega, pressure, temperature, mixing=0):
+    r"""Calculate w from omega assuming hydrostatic conditions.
+
+    This function converts vertical velocity with respect to pressure
+    :math:`\left(\omega = \frac{Dp}{Dt}\right)` to that with respect to height
+    :math:`\left(w = \frac{Dz}{Dt}\right)` assuming hydrostatic conditions on
+    the synoptic scale. By Equation 7.33 in [Hobbs2006]_,
+
+    .. math: \omega \simeq -\rho g w
+
+    so that
+
+    .. math w \simeq \frac{- \omega}{\rho g}
+
+    Density (:math:`\rho`) is calculated using the :func:`density` function,
+    from the given pressure and temperature. If `mixing` is given, the virtual
+    temperature correction is used, otherwise, dry air is assumed.
+
+    Parameters
+    ----------
+    omega: `pint.Quantity`
+        Vertical velocity in terms of pressure
+    pressure: `pint.Quantity`
+        Total atmospheric pressure
+    temperature: `pint.Quantity`
+        Air temperature
+    mixing: `pint.Quantity`, optional
+        Mixing ratio of air
+
+    Returns
+    -------
+    `pint.Quantity`
+        Vertical velocity in terms of height (in meters / second)
+
+    See Also
+    --------
+    density, vertical_velocity_pressure
+
+    """
+    rho = density(pressure, temperature, mixing)
+    return (omega / (- g * rho)).to('m/s')
